﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using Infragistics.Controls.Charts;
using System.Windows.Browser;
using System.IO;
using System.Text;
using System.Collections.ObjectModel;
using System.Windows.Threading;
using System.ComponentModel;
using System.Windows.Data;
using System.Globalization;
using System.Windows.Media.Imaging;

namespace XamWebDataChartSLApp
{
    public partial class MainPage : UserControl
    {
        private ObservableCollection<ChannelDataGroup> historicalData = new ObservableCollection<ChannelDataGroup>();
        private const int queueCapacity = 360;
        private Dictionary<int, ObservableQueue<ChannelData>> liveData = new Dictionary<int, ObservableQueue<ChannelData>>();
        private ObservableCollection<ChannelLookup> channelNames = new ObservableCollection<ChannelLookup>();

        public DataMode dataMode { get; set; }

        public MainPage()
        {
            InitializeComponent();
            dataMode = DataMode.Live;
            this.LayoutUpdated += new EventHandler(MainPage_LayoutUpdated);
            xmDataChart.PropertyUpdated += new PropertyUpdatedEventHandler(xmDataChart_PropertyUpdated);
            this.channelNames.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(channelNames_CollectionChanged);
        }

        void channelNames_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            // add/remove series and data for the specific channel
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add && e.NewItems != null)
            {
                foreach (ChannelLookup channel in e.NewItems)
                {
                    // add the live data queue
                    AddLiveChannelQueue(channel.ChannelIndex);
                    AddHistoricalChannelData(channel.ChannelIndex);
                }
            }
            else if (e.OldItems != null)
            {
                // remove series
                foreach (ChannelLookup channel in e.OldItems)
                {
                    // remove data from live queue and then refresh the graph since this is not an observable collection
                    liveData.Remove(channel.ChannelIndex);
                    RemoveLayer(channel.ChannelName);

                    // just delete the historical data and the graph will automatically update
                    // Have to use linq since Exists() and FirstOrDefault() is not available in SL and 
                    // First() blows up if it is not found
                    var group = (from curr in historicalData
                                    where curr.index == channel.ChannelIndex
                                    select curr);

                    if (group != null && group.Count() > 0)
                        historicalData.Remove(group.First());
                }
            }
        }
        private void AddSeries(int channelIndex)
        {
            ChannelLookup lookup = channelNames.First(c => c.ChannelIndex == channelIndex);

            // Set the interval 
            CategoryXAxis xaxis = xmDataChart.Axes["XAxis"] as CategoryXAxis;
            //xaxis.Interval = 72;

            // create new Y axis
            NumericYAxis yaxis = new NumericYAxis();
            AxisLabelSettings labelsettings = new AxisLabelSettings();
            labelsettings.Visibility = System.Windows.Visibility.Collapsed;
            labelsettings.Location = AxisLabelsLocation.InsideLeft;
            yaxis.LabelSettings = labelsettings;
            yaxis.Style = Resources["NumericYAxisStyle"] as Style;
            yaxis.MinimumValue = lookup.Min;
            yaxis.MaximumValue = lookup.Max;
            yaxis.Name = lookup.ChannelName;
            xmDataChart.Axes.Add(yaxis);

            DoubleCollection dashArray = new DoubleCollection();
            dashArray.Add(3.0);
            dashArray.Add(1.0);

            // Create a new series
            StepLineSeries series = new StepLineSeries();
            series.Title = lookup.ChannelName;
            series.Name = String.Format("series_{0}", lookup.ChannelName);
            series.MarkerType = MarkerType.None;
            series.ValueMemberPath = "Value";
            series.XAxis = xaxis;
            series.XAxis.ItemsSource = liveData[channelIndex];
            series.YAxis = yaxis;
            //series.DashArray = dashArray;
            series.ItemsSource = liveData[channelIndex];
            xmDataChart.Series.Add(series);
        }
        private void AddHistoricalSeries(int channelIndex)
        {
            ChannelLookup lookup = channelNames.First(c => c.ChannelIndex == channelIndex);
            List<ChannelData> data = historicalData.First(d => d.index == channelIndex).ChannelDataList;
            CategoryXAxis xaxis = xmDataChart.Axes["XAxis"] as CategoryXAxis;

            //xaxis.Interval = DetermineXAxisInterval(data, 1);

            // create new Y axis
            NumericYAxis yaxis = new NumericYAxis();
            AxisLabelSettings labelsettings = new AxisLabelSettings();
            labelsettings.Visibility = System.Windows.Visibility.Collapsed;
            labelsettings.Location = AxisLabelsLocation.InsideLeft;
            yaxis.LabelSettings = labelsettings;
            yaxis.Style = Resources["NumericYAxisStyle"] as Style;
            yaxis.MinimumValue = lookup.Min;
            yaxis.MaximumValue = lookup.Max;
            yaxis.Name = lookup.ChannelName;
            xmDataChart.Axes.Add(yaxis);

            // create a stepline series
            StepLineSeries series = new StepLineSeries();
            series.Title = lookup.ChannelName;
            series.Name = String.Format("series_{0}", lookup.ChannelName);
            series.MarkerType = MarkerType.None;
            series.ValueMemberPath = "Value";
            series.XAxis = xaxis;
            series.XAxis.ItemsSource = data;
            series.YAxis = yaxis;
            series.ItemsSource = data;
            xmDataChart.Series.Add(series);
        }
        private void LoadData()
        {
            if (channelNames.Count > 0)
                LoadData(channelNames.Select(cn => cn.ChannelIndex).ToList());
        }
        private void LoadData(List<int> indexes)
        {
            List<ChannelData> sampledata = GenerateSampleData(indexes);
            foreach (var channelindex in sampledata.Select(c => c.ChannelIndex).Distinct())
            {
                historicalData.Add(new ChannelDataGroup() { index = channelindex, ChannelDataList = sampledata.Where(c => c.ChannelIndex == channelindex).ToList() });
                AddHistoricalSeries(channelindex);
            }
        }
        private void AddToQueue(ObservableQueue<ChannelData> queue, ChannelData channelData)
        {
            if (queue.Count >= queueCapacity)
                queue.Dequeue();
            queue.Enqueue(channelData);
        }
        private void AddLiveChannelQueue(int channelIndex)
        {
            if (!liveData.ContainsKey(channelIndex))
            {
                // create the queue
                ObservableQueue<ChannelData> tempqueue = new ObservableQueue<ChannelData>(queueCapacity);
                

                // pre pop the collection with nulls
                DateTime past = DateTime.Now;
                //int increment = 360 / queueCapacity;
                for (int x = 0; x < queueCapacity; x++)
                {
                    tempqueue.Enqueue(new ChannelData() { TimeStamp = past.Subtract(new TimeSpan(0, 0, 360 - x)), Value = null });
                }

                liveData.Add(channelIndex, tempqueue);

                if (dataMode == DataMode.Live)
                {
                    // Create the series mapping
                    AddSeries(channelIndex);
                }
            }
        }
        private void AddHistoricalChannelData(int index)
        {
            if (dataMode == DataMode.Historical)
            {
                LoadData(new List<int>() { index });
            }
        }
        private void RemoveLayer(string name)
        {
            xmDataChart.Series.Remove(xmDataChart.Series[String.Format("series_{0}", name)]);
            xmDataChart.Axes.Remove(xmDataChart.Axes[name]);
        }
        private void UpdateQueueData(Viper viperunit)
        {
            if (viperunit == null)
                return;

            // Loop through each channel data item and add it to its respective queue but 
            // only if that index is in the channelnames. That list contains all the names/indexes
            // that the graph should be plotting.
            foreach (var reading in viperunit.ChannelDataList.Where(d => channelNames.Count(cn => cn.ChannelIndex == d.ChannelIndex) > 0).OrderBy(d => d.TimeStamp))
            {
                // Create the queue and add the layer to the graph if it is not already there
                AddLiveChannelQueue(reading.ChannelIndex);

                // Add new item to the queue
                ObservableQueue<ChannelData> queue = liveData[reading.ChannelIndex];
                if (queue.Count == 0 || reading.TimeStamp > queue.Last().TimeStamp)
                {
                    // Add new entry
                    AddToQueue(queue, reading);
                }
            }
        }
        private void SetHorizontalZoom(bool enable)
        {
            xmDataChart.WindowScaleHorizontal = 1;
            if (enable)
            {
                xmDataChart.HorizontalZoombarVisibility = System.Windows.Visibility.Visible;
                xmDataChart.HorizontalZoomable = true;
            }
            else
            {
                xmDataChart.HorizontalZoombarVisibility = System.Windows.Visibility.Collapsed;
                xmDataChart.HorizontalZoomable = false;
            }
        }
        private void ToggleChartView()
        {
            foreach (var series in xmDataChart.Series)
            {
                xmDataChart.Axes.Remove(xmDataChart.Axes[series.Title.ToString()]);
            }
            xmDataChart.Series.Clear();


            if (dataMode == DataMode.Historical) // set to historical mode
            {
                // Make sure the horizontal zoom bar is visible
                SetHorizontalZoom(true);
                LoadData();
            }
            else // set to Live mode
            {
                // turn off the horizontal zoom bar when in live view
                SetHorizontalZoom(false);
                CategoryXAxis axis = xmDataChart.Axes["XAxis"] as CategoryXAxis;
                foreach (var channel in liveData)
                {
                    AddSeries(channel.Key);
                }
            }
        }

        #region Get Crosshair values
        private void xmDataChart_PropertyUpdated(object sender, Infragistics.Controls.Charts.PropertyUpdatedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "WindowRect":
                    if (e.NewValue != null && e.NewValue is Rect)
                    {
                        Rect oldview = (Rect)e.OldValue;
                        Rect newview = (Rect)e.NewValue;
                        if (oldview.Width != newview.Width)
                        {
                            // we zoomed so reset the x axis interval
                            //((CategoryXAxis)xmDataChart.Axes["XAxis"]).Interval = DetermineXAxisInterval(historicalData[0].ChannelDataList, newview.Width);
                        }
                    }
                    break;
            }
        }
        #endregion

        #region User events
        private void btnHistory_Click(object sender, RoutedEventArgs e)
        {
            // toggle the datamode
            dataMode = dataMode == DataMode.Live ? DataMode.Historical : DataMode.Live;

            string path = dataMode == DataMode.Live ? "images/clock_play.png" : "images/clock_stop.png";
            Button button = sender as Button;
            Image image = button.Content as Image;
            image.Source = new BitmapImage(new Uri(path, UriKind.Relative));
            ToolTipService.SetToolTip(button, dataMode == DataMode.Live ? "Show History" : "Show Live Data");
            ToggleChartView();
        }

        #endregion

        #region Sample Data generation
        DispatcherTimer t = new DispatcherTimer();

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            // Timer is strictly for sample "live" data
            t.Interval = new TimeSpan(0, 0, 3);
            t.Tick += new EventHandler(UpdateDataDelegate);
            t.Start();
            //SetupChart();
            //LoadData();
        }
        private void UpdateDataDelegate(object sender, EventArgs e)
        {
            Random r = new Random();
            List<ChannelData> channeldata = new List<ChannelData>();
            channeldata.Add(new ChannelData() { ChannelIndex = 1, Value = r.Next(0, 100), TimeStamp = DateTime.Now });
            channeldata.Add(new ChannelData() { ChannelIndex = 2, Value = r.Next(40, 80), TimeStamp = DateTime.Now });

            Viper viperunit = new Viper()
            {
                SerialNumber = "1234",
                ViperId = 1,
                Status = 1,
                ChannelDataList = channeldata
            };

            UpdateQueueData(viperunit);
        }
        private List<ChannelData> GenerateSampleData(List<int> indexes)
        {
            Random r = new Random();

            DateTime time = DateTime.Now;
            List<ChannelData> dataList = new List<ChannelData>();
            foreach (int index in indexes)
            {
                for (int x = 0; x < 100; x++)
                {
                    dataList.Add(new ChannelData() { ChannelIndex = index, Value = r.Next(0, 100), TimeStamp = time.AddSeconds(x - 100) });
                }
            }
            return dataList;
        }
        void MainPage_LayoutUpdated(object sender, EventArgs e)
        {
            // Put the chart into live mode
            // add the two names into the names list
            // start the timer
            if (channelNames.Count < 2)
            {
                dataMode = DataMode.Live;
                channelNames.Add(new ChannelLookup() { ChannelIndex = 1, ChannelName = "SvO2", Max = 100, Min = 0, Units = "%" });
                channelNames.Add(new ChannelLookup() { ChannelIndex = 2, ChannelName = "HCT", Max = 100, Min = 0, Units = "%" });
                SetHorizontalZoom(false);
            }
        }
        #endregion
    }
}